@@ -75,7 +75,7 @@ def check_instance(directory):
     pid = open(pidfile, "r").readline()
     if pid:
         p = os.system("/bin/ps %s >/dev/null" % pid)
-        p_name = os.popen("/bin/ps -f %s" % pid).read()
+        p_name = os.popen("/bin/ps -p %s" % pid).read()
         if p == 0 and "gtg" in p_name:
             print _("gtg is already running!")
             d=dbus.SessionBus().get_object(CoreConfig.BUSNAME,\
