@@ -18,7 +18,6 @@ import gtk
 import os, sys
 #from time import sleep
 
-from xdg.BaseDirectory import *
 #from configobj import ConfigObj
 
 import Geoclue
@@ -219,7 +218,7 @@ class geolocalizedTasks:
     def is_configurable(self):
         return True
     
-    def configure_dialog(self, plugin_api):
+    def configure_dialog(self, plugin_api, manager_dialog):
         self.on_geolocalized_preferences(plugin_api)
     
     def location_changed(self):
