@@ -14,7 +14,12 @@
 # You should have received a copy of the GNU General Public License along with
 # this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import sys,os
+import sys
+import os
 sys.path.insert(0,os.getcwd())
 
-from hamster import hamsterPlugin
\ No newline at end of file
+from hamster import hamsterPlugin
+
+
+#suppress pyflakes warning (given by make lint)
+if False == True: hamsterPlugin()
