@@ -37,6 +37,8 @@ COL_CDATE_STR = 5
 COL_DLEFT     = 6
 COL_TAGS      = 7
 COL_LABEL     = 9
+COL_SDATE     = 10
+COL_DUE       = 11
 
 class TaskTreeModel(gtk.GenericTreeModel):
 
@@ -50,6 +52,8 @@ class TaskTreeModel(gtk.GenericTreeModel):
         str,\
         gobject.TYPE_PYOBJECT,\
         str,\
+        str,\
+        str,\
         str)
 
     def __init__(self, requester):
@@ -110,10 +114,14 @@ class TaskTreeModel(gtk.GenericTreeModel):
             return task
         elif column == COL_TITLE:
             return saxutils.escape(task.get_title())
+        elif column == COL_SDATE:
+            return task.get_start_date().to_readable_string()
         elif column == COL_DDATE:
-            return str(task.get_due_date())
+            return task.get_due_date().to_readable_string()
+        elif column == COL_DUE:
+            return task.get_due_date().to_readable_string()
         elif column == COL_CDATE:
-            return str(task.get_closed_date())
+            return task.get_closed_date().to_readable_string()
         elif column == COL_CDATE_STR:
             if task.get_status() == Task.STA_DISMISSED:
                 date = "<span color='#AAAAAA'>" +\
@@ -124,7 +132,9 @@ class TaskTreeModel(gtk.GenericTreeModel):
         elif column == COL_DLEFT:
             return task.get_days_left()
         elif column == COL_TAGS:
-            return task.get_tags()
+            tags = task.get_tags()
+            tags.sort(key = lambda x: x.get_name())
+            return tags
         elif column == COL_LABEL:
             if task.get_status() == Task.STA_ACTIVE:
                 count = self._count_active_subtasks_rec(task)
@@ -355,6 +365,7 @@ class ActiveTaskTreeView(TaskTreeView):
     def __init__(self):
         TaskTreeView.__init__(self)
         self._init_tree_view()
+        self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
 
         # Drag and drop
         self.enable_model_drag_source(\
@@ -378,6 +389,9 @@ class ActiveTaskTreeView(TaskTreeView):
         self.connect('drag_drop', self.on_drag_drop)
         self.connect('drag_data_get', self.on_drag_data_get)
         self.connect('drag_data_received', self.on_drag_data_received)
+        self.connect('button_press_event', self.on_button_press)
+        self.connect('button_release_event', self.on_button_release)
+        self.defer_select = False
 
     def _init_tree_view(self):
         # Tag column
@@ -407,30 +421,43 @@ class ActiveTaskTreeView(TaskTreeView):
         title_col.set_cell_data_func(render_text, self._celldatafunction)
         self.append_column(title_col)
         self.columns.insert(COL_TITLE, title_col)
+        self.set_search_column(COL_TITLE)
 
-        # Due date column
+        # Start date column
+        sdate_col   = gtk.TreeViewColumn()
+        render_text = gtk.CellRendererText()
+        sdate_col.set_title(_("Start date"))
+        sdate_col.pack_start(render_text, expand=False)
+        sdate_col.add_attribute(render_text, "markup", COL_SDATE)
+        sdate_col.set_resizable(False)
+        sdate_col.set_sort_column_id(COL_SDATE)
+        sdate_col.set_cell_data_func(render_text, self._celldatafunction)
+        self.append_column(sdate_col)
+        self.columns.insert(COL_SDATE, sdate_col)
+
+        # Due column
         ddate_col   = gtk.TreeViewColumn()
         render_text = gtk.CellRendererText()
-        ddate_col.set_title(_("Due date"))
+        ddate_col.set_title(_("Due"))
         ddate_col.pack_start(render_text, expand=False)
-        ddate_col.add_attribute(render_text, "markup", COL_DDATE)
+        ddate_col.add_attribute(render_text, "markup", COL_DUE)
         ddate_col.set_resizable(False)
         ddate_col.set_sort_column_id(COL_DDATE)
         ddate_col.set_cell_data_func(render_text, self._celldatafunction)
         self.append_column(ddate_col)
-        self.columns.insert(COL_DDATE, ddate_col)
+        self.columns.insert(COL_DUE, ddate_col)
 
         # days left
-        dleft_col   = gtk.TreeViewColumn()
-        render_text = gtk.CellRendererText()
-        dleft_col.set_title(_("Days left"))
-        dleft_col.pack_start(render_text, expand=False)
-        dleft_col.add_attribute(render_text, "markup", COL_DLEFT)
-        dleft_col.set_resizable(False)
-        dleft_col.set_sort_column_id(COL_DLEFT)
-        dleft_col.set_cell_data_func(render_text, self._celldatafunction)
-        self.append_column(dleft_col)
-        self.columns.insert(COL_DLEFT, dleft_col)
+#        dleft_col   = gtk.TreeViewColumn()
+#        render_text = gtk.CellRendererText()
+#        dleft_col.set_title(_("Days left"))
+#        dleft_col.pack_start(render_text, expand=False)
+#        dleft_col.add_attribute(render_text, "markup", COL_DLEFT)
+#        dleft_col.set_resizable(False)
+#        dleft_col.set_sort_column_id(COL_DLEFT)
+#        dleft_col.set_cell_data_func(render_text, self._celldatafunction)
+#        self.append_column(dleft_col)
+#        self.columns.insert(COL_DLEFT, dleft_col)
 
         # Global treeview properties
         self.set_property("expander-column", title_col)
@@ -438,6 +465,31 @@ class ActiveTaskTreeView(TaskTreeView):
         self.set_rules_hint(False)
 
     ### DRAG AND DROP ########################################################
+    def on_button_press(self, widget, event):
+        # Here we intercept mouse clicks on selected items so that we can
+        # drag multiple items without the click selecting only one
+        target = self.get_path_at_pos(int(event.x), int(event.y))
+        if (target 
+           and event.type == gtk.gdk.BUTTON_PRESS
+           and not (event.state & (gtk.gdk.CONTROL_MASK|gtk.gdk.SHIFT_MASK))
+           and self.get_selection().path_is_selected(target[0])):
+               # disable selection
+               self.get_selection().set_select_function(lambda *ignore: False)
+               self.defer_select = target[0]
+            
+    def on_button_release(self, widget, event):
+        # re-enable selection
+        self.get_selection().set_select_function(lambda *ignore: True)
+        
+        target = self.get_path_at_pos(int(event.x), int(event.y))    
+        if (self.defer_select and target 
+           and self.defer_select == target[0]
+           and not (event.x==0 and event.y==0)): # certain drag and drop 
+                                                 # operations still have path
+               # if user didn't drag, simulate the click previously ignored
+               self.set_cursor(target[0], target[1], False)
+            
+        self.defer_select=False
 
     def on_drag_drop(self, treeview, context, selection, info, timestamp):
         self.emit_stop_by_name('drag_drop')
@@ -447,8 +499,9 @@ class ActiveTaskTreeView(TaskTreeView):
         the parent task and the id of the selected task is passed to the
         destination"""
         treeselection = treeview.get_selection()
-        model, iter = treeselection.get_selected()
-        iter_str = model.get_string_from_iter(iter)
+        model, paths = treeselection.get_selected_rows()
+        iters = [model.get_iter(path) for path in paths]
+        iter_str = ','.join([model.get_string_from_iter(iter) for iter in iters])
         selection.set('gtg/task-iter-str', 0, iter_str)
         return
 
@@ -489,12 +542,14 @@ class ActiveTaskTreeView(TaskTreeView):
             par_iter_tasktree = None
 
         # Get dragged iter as a TaskTreeModel iter
-        drag_iter = model.get_iter_from_string(selection.data)
-        drag_iter_filter   =\
-            model.convert_iter_to_child_iter(None, drag_iter)
-        drag_iter_tasktree =\
-            model_filter.convert_iter_to_child_iter(drag_iter_filter)
-        tasktree_model.move_task(par_iter_tasktree, drag_iter_tasktree)
+        iters = selection.data.split(',')
+        for iter in iters:
+            drag_iter = model.get_iter_from_string(iter)
+            drag_iter_filter   =\
+                model.convert_iter_to_child_iter(None, drag_iter)
+            drag_iter_tasktree =\
+                model_filter.convert_iter_to_child_iter(drag_iter_filter)
+            tasktree_model.move_task(par_iter_tasktree, drag_iter_tasktree)
 
         self.emit_stop_by_name('drag_data_received')
 
@@ -541,6 +596,7 @@ class ClosedTaskTreeView(TaskTreeView):
         title_col.set_sort_column_id(COL_TITLE)
         self.append_column(title_col)
         self.columns.insert(COL_TITLE, title_col)
+        self.set_search_column(COL_TITLE)
         
         self.set_show_expanders(False)
 
