@@ -20,9 +20,9 @@
 
 #This is a class taken originally from
 #http://trac.atzm.org/index.cgi/wiki/PyGTK
-#It was in Japanese and I didn't understood anything but the code.
+#It was in Japanese and I didn't understand anything but the code.
 
-#This class implement a gtk.TextView but with many other features
+#This class implements a gtk.TextView but with many other features
 #like hyperlink and others stuffs special for GTG
 #
 #For your information, a gtkTextView always contains a gtk.TextBuffer which
@@ -538,7 +538,6 @@ class TaskView(gtk.TextView):
     #Detect URL in the tasks
     #It's ugly...
     def _detect_url(self,buff,start,end) :
-        #firtst, we remove the old tags :
         #subt_list = self.get_subtasks()
         #First, we remove the olds tags
         tag_list = []
@@ -587,7 +586,7 @@ class TaskView(gtk.TextView):
                 
         
 
-    #Detect tags in buff in the regio between start iter and end iter
+    #Detect tags in buff in the region between start iter and end iter
     def _detect_tag(self,buff,start,end) :
         # Removing already existing tag in the current selection
         # out of the tag table
@@ -596,7 +595,7 @@ class TaskView(gtk.TextView):
         old_tags = []
         new_tags = []
         #We must be strictly < than the end_offset. If not, we might
-        #found the beginning of a tag on the nextline
+        #find the beginning of a tag on the nextline
         while (it.get_offset() < end.get_offset()) and (it.get_char() != '\0'):
             if it.begins_tag() :
                 tags = it.get_toggled_tags(True)
@@ -687,7 +686,7 @@ class TaskView(gtk.TextView):
             to_return = True
         return to_return
         
-    #When the user remove a selection, we remove subtasks and @tags
+    #When the user removes a selection, we remove subtasks and @tags
     #from this selection
     def _delete_range(self,buff,start,end) :
 #        #If we are at the beginning of a mark, put this mark at the end
@@ -864,16 +863,26 @@ class TaskView(gtk.TextView):
                 if t.get_data('is_indent') :
                     line += 1
                     startl = self.buff.get_iter_at_line(line)
-                    found = False
+                    if line < self.buff.get_line_count() :
+                        found = False
             if found :
                 itera = startl 
+                
+        #if the last line is indented, then insert a new line
+        #at the end
+        if line == self.buff.get_line_count() :
+            itera.forward_to_line_end()
+            mark = self.buff.create_mark(None,itera,True)
+            self.buff.insert(itera,"\n")
+            itera = self.buff.get_iter_at_mark(mark)
+            self.buff.delete_mark(mark)
                     
         #If we are not on the end of line, go there
         #but if we are at the start of line, then create the subtask
         #before the current line
         enter = True
         if itera.starts_line() :
-            mark = self.buff.create_mark("temp",itera,True)
+            mark = self.buff.create_mark(None,itera,True)
             self.buff.insert(itera,"\n")
             itera = self.buff.get_iter_at_mark(mark)
             self.buff.delete_mark(mark)
@@ -914,7 +923,7 @@ class TaskView(gtk.TextView):
             itera = buff.get_iter_at_mark(temp_mark)
             buff.move_mark_by_name("/%s"%subtid,itera)
         buff.delete_mark(temp_mark)
-        #The mark has right gravity but because we putted it on the left
+        #The mark has right gravity but because we put it on the left
         #of the newly inserted \n, it will not move anymore.
         
         itera = buff.get_iter_at_mark(end)
@@ -974,6 +983,9 @@ class TaskView(gtk.TextView):
         
         #First, we analyse the selection to put in our own
         #GTG clipboard a selection with description of subtasks
+        bounds =  self.buff.get_selection_bounds()
+        if not bounds:
+            return
         start, stop =  self.buff.get_selection_bounds()
         
         self.clipboard.copy(start,stop,bullet=self.bullet1)
@@ -995,25 +1007,29 @@ class TaskView(gtk.TextView):
         #else, that we can empty it.
         our_paste = self.clipboard.paste_text()
         if our_paste != None and clip.wait_for_text() == our_paste :
-            #we handle ourselves the pasting
-            self.stop_emission("paste_clipboard")
+            #first, we delete the current selection
+            self.buff.delete_selection(False,True)
             for line in self.clipboard.paste():
                 if line[0] == 'text':
                     self.buff.insert_at_cursor(line[1])
-                elif line[0] == 'subtask':
-                    self.new_subtask_callback(tid=line[1])
+                if line[0] == 'subtask':
+                    tid=line[1]
+                    self.new_subtask_callback(tid=tid)
                     mark = self.buff.get_insert()
                     line_nbr = self.buff.get_iter_at_mark(mark).get_line()
                     #we must paste the \n before inserting the subtask
                     #else, we will start another subtask
                     self.buff.insert_at_cursor("\n")
-                    self.write_subtask(self.buff,line_nbr,line[1])
+                    self.write_subtask(self.buff,line_nbr,tid)
+
+            #we handle ourselves the pasting
+            self.stop_emission("paste_clipboard")
 
         else:
             #we keep the normal pasting by not interupting the signal
             self.clipboard.clear()
         
-    #Function called each time the user input a letter   
+    #Function called each time the user inputs a letter   
     def _insert_at_cursor(self, tv, itera, tex, leng) :
         #We don't paste the bullet
         if tex.strip() != self.bullet1 :
@@ -1066,6 +1082,13 @@ class TaskView(gtk.TextView):
                 #Because it's the title
                 if line_nbr > 0 :
                     line = start_line.get_slice(end_line)
+                    #the part after the enter
+                    realend = end_line.copy()
+                    restofline = None
+                    if not realend.ends_line():
+                        realend.forward_to_line_end()
+                        restofline = end_line.get_slice(realend)
+                        restofline.strip()
                     
                     #If indent is 0, We check if we created a new task
                     #the "-" might be after a space
@@ -1075,7 +1098,17 @@ class TaskView(gtk.TextView):
                             line = line.lstrip(' -')
                             end_i = self.__newsubtask(self.buff,line,line_nbr)
                             #Here, we should increment indent level
-                            self.insert_indent(self.buff,end_i,1,enter=True)
+                            #If we inserted enter in the middle of a line
+                            if restofline and restofline.strip() != "" :
+                                #it means we have two subtask to create
+                                if self.buff.get_line_count() > line_nbr+1:
+                                    #but don't merge with the next line
+                                    itera = self.buff.get_iter_at_line(line_nbr+1)
+                                    self.buff.insert(itera,"\n\n")
+                                self.__newsubtask(self.buff,restofline,\
+                                                            line_nbr+1)
+                            else:
+                                self.insert_indent(self.buff,end_i,1,enter=True)
                             tv.emit_stop_by_name('insert-text')
                         else :
                             self.buff.insert(itera,"\n")
@@ -1084,13 +1117,25 @@ class TaskView(gtk.TextView):
                     #Then, if indent > 0, we increment it
                     #First step : we preserve it.
                     else :
-                        if not line.lstrip("%s "%self.bullet1) :
-                            self.deindent(itera,newlevel=0)
-                            tv.emit_stop_by_name('insert-text')
-                            
+                        if not line.lstrip("%s "%self.bullet1):
+                            #if we didn't write a task, we remove the indent
+                            #we check if the iterator is well at the end of 
+                            #the line
+                            if end_line.ends_line():
+                                self.deindent(itera,newlevel=0)
+                            #else, it means that we pressed enter before 
+                            #a subtask title
+                            else :
+                                #we first put the subtask one line below
+                                itera2 = self.buff.get_iter_at_line(line_nbr)
+                                self.buff.insert(itera2,"\n")
+                                #and increment the new white line
+                                itera2 = self.buff.get_iter_at_line(line_nbr)
+                                self.insert_indent(self.buff,itera2,current_indent,enter=False)
                         elif current_indent == 1 :
                             self.insert_indent(self.buff,itera,current_indent)
-                            tv.emit_stop_by_name('insert-text')
+                        #we stop the signal in all cases
+                        tv.emit_stop_by_name('insert-text')
                     #Then we close the tag tag
                     if closed_tag :
                         insert_mark = self.buff.get_mark("insert_point")
@@ -1111,10 +1156,21 @@ class TaskView(gtk.TextView):
             elif tex :
                 #We are on an indented line without subtask ? Create it !
                 if current_indent > 0 and not subtask_nbr :
-                    #self.__newsubtask(self.buff,tex,line_nbr, level=current_indent)
-                    anchor = self.new_subtask_callback(tex)
-                    self.buff.create_mark(anchor,itera,True)
-                    self.buff.create_mark("/%s"%anchor,itera,False)
+                    if itera.starts_line():
+                        #we are at the start of an existing subtask
+                        #we simply move that subtask down
+                        self.buff.insert(itera,"\n")
+                        itera2 = self.buff.get_iter_at_line(line_nbr)
+                        self.buff.insert(itera2,tex)
+                        itera3 = self.buff.get_iter_at_line(line_nbr)
+                        itera3.forward_to_line_end()
+                        self.buff.place_cursor(itera3)
+                        tv.emit_stop_by_name('insert-text')
+                    else:
+                        #self.__newsubtask(self.buff,tex,line_nbr, level=current_indent)
+                        anchor = self.new_subtask_callback(tex)
+                        self.buff.create_mark(anchor,itera,True)
+                        self.buff.create_mark("/%s"%anchor,itera,False)
             self.insert_sigid = self.buff.connect('insert-text', self._insert_at_cursor)
             self.connect('key_press_event', self._keypress)
             self.modified_sigid = self.buff.connect("changed" , self.modified)
@@ -1170,7 +1226,7 @@ class TaskView(gtk.TextView):
         self.buff.disconnect(self.insert_sigid)
         insert_mark = self.buff.get_insert()
         insert_iter = self.buff.get_iter_at_mark(insert_mark)
-        #All that craps to find if we are at the end of an indent tag
+        #All this crap to find if we are at the end of an indent tag
         if insert_iter.ends_tag() :
             for t in insert_iter.get_toggled_tags(False) :
                 if t.get_data('is_indent') :
@@ -1182,7 +1238,7 @@ class TaskView(gtk.TextView):
         self.insert_sigid = self.buff.connect('insert-text', \
                                                self._insert_at_cursor)
 
-    #The mouse is moving. We must change it to a hand when hovering a link
+    #The mouse is moving. We must change it to a hand when hovering over a link
     def _motion(self, view, ev):
         window = ev.window
         x, y, _ = window.get_pointer()
